import os
import shutil

from fasttemplates.constants import WORKING_DIR


def copy_files(src_dir, dst_dir, ignore_dirs=None):
    if ignore_dirs is None:
        ignore_dirs = []

    for item in os.listdir(src_dir):
        if item in ignore_dirs:
            continue

        src_item = os.path.join(src_dir, item)
        dst_item = os.path.join(dst_dir, item)

        if os.path.isdir(src_item):
            os.makedirs(dst_item, exist_ok=True)
            copy_files(src_item, dst_item, ignore_dirs)
        else:
            shutil.copy2(src_item, dst_item)

def cout_settings_properties() -> int:
    settings_path = os.path.join(WORKING_DIR, "app\\settings.py")
    with open(settings_path, "r") as f:
        lines = f.readlines()
    
    class_def_index = next(i for i, line in enumerate(
        lines) if SETTINGS_CLASS_DEFINITION in line)
    
    return len([line for line in lines if line.strip() and not line.strip().startswith('#')])+1

def add_property_to_settings(_K: str, _T: type, _V: str) -> None:
    settings_path = os.path.join(WORKING_DIR, SETTINGS_DIR)
    with open(settings_path, "r") as f:
        lines = f.readlines()
    
    NUMBER_OF_PROPERTIES = cout_settings_properties()
    lines.insert(NUMBER_OF_PROPERTIES, f'    {_K}: {_T.__name__} = {_V}\n')

    with open(settings_path, "w") as f:
        f.writelines(lines)

def add_mongo_uri_to_settings():
    add_property_to_settings('MONGO_URI', str, "'mongodb://localhost:27017'")

def add_redis_uri_to_settings():
    add_property_to_settings('REDIS_URI', str, "'redis://localhost:6379/0'")

def on_rm_error(func, path, exc_info) -> None:
    "Error handler for `shutil.rmtree`"
    os.chmod(path, stat.S_IWRITE)
    os.unlink(path)

def clone_and_copy(repo_url: str) -> None:
    with tempfile.TemporaryDirectory() as tmpdirname:
        os.system(f'git clone {repo_url} {tmpdirname}')
        shutil.rmtree(os.path.join(tmpdirname, '.git'), onerror=on_rm_error)
        copy_files(tmpdirname, WORKING_DIR)


import os

MONGO_DIR_NAME = 'mongo'
REDIS_DIR_NAME = 'redis'

MONGO_PROMPT = "MongoDB database"
REDIS_PROMPT = "Redis database"

WORKING_DIR = os.getcwd()
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

"BASE LAYOUT"
BASE_LAYOUT = os.path.join(BASE_DIR, "templates\\baselayout")

"SETTINGS LOCATION"
SETTINGS_DIR = "app\\settings.py"

SETTINGS_CLASS_DEFINITION = 'class Settings(BaseSettings):'

"EXTRAS"
DB_DIR = "app\\db"
DB_TREE = os.path.join(BASE_DIR, "templates\\extras\\db")
REDIS_CACHE_TREE = os.path.join(BASE_DIR, "templates\\extras\\cache\\redis")

"Repositories"
FASTAPI_FS = 'FastAPI Fullstack Template by Tiangolo'
FASTAPI_DJANGO_ORM = 'FastAPI with Django ORM by Uhttred'
FASTAPI_FULLSTACK = 'https://github.com/tiangolo/full-stack-fastapi-template.git'
FASTAPI_WITH_DJANGO_ORM = 'https://github.com/uhttred/fastapi-with-django-orm-template.git'

docker_compose_dict = {
    'version': '3.8',
    'services': {
      'backend': {
        'build': '.',
        'command': 'uvicorn app.main:app --host 0.0.0.0 --port 80 --reload --log-level debug',
        'volumes': ['.:/app'],
        'ports': ['8000:80'],
        'env_file': ['.env'],
        },
    },
    'volumes':{
    }
}
mongo_service = {
  'db': {
    'image': 'mongo:latest',
    'ports': ['27017:27017'],
    'volumes': ['mongodb_data_container:/data/db']
  }
}
redis_service = {
  'redis': {
    'image': 'redis:latest',
    'ports': ['6379:6379'],
    'volumes': ['redis_data_container:/data/db']
  }
}
mongo_volume = {
  'mongodb_data_container': {}
}
redis_volume = {
  'redis_data_container': {}
}


import os

import typer
import questionary
import tempfile
import shutil
import stat
import yaml

from fasttemplates.constants import *
from fasttemplates.utils import *

app = typer.Typer()


def custom() -> None:
    options = questionary.checkbox("What do you want to add?", choices=[
        REDIS_PROMPT, MONGO_PROMPT]).ask()
    mongo_present = MONGO_PROMPT in options
    redis_present = REDIS_PROMPT in options
    add_base_layout()

    if mongo_present:
        add_mongo()
        print("MongoDB database added")
    if redis_present:
        add_redis()
        print("Redis database added")
        cache = questionary.confirm("Do you want to use Redis as cache?").ask()
        if cache:
            add_cache_with_redis()
            print("Redis cache added")
    
    create_docker_compose()


def popular() -> None:
    "Clones the popular template from the github repository"

    optionLambdas = {
        FASTAPI_FS: lambda: clone_and_copy(FASTAPI_FULLSTACK),
        FASTAPI_DJANGO_ORM: lambda: clone_and_copy(FASTAPI_WITH_DJANGO_ORM),
    }

    option = questionary.select("Which popular template do you want to clone?", choices=[
        FASTAPI_FS, FASTAPI_DJANGO_ORM]).ask()

    optionLambdas[option]()

    print("Popular template added")

@app.command()
def main() -> None:
    popular_or_simplified_functions = {
        'Popular': popular,
        'Custom': custom
    }
    popular_or_simplified = questionary.select("Popular templates or custom templates?", choices=[
        'Popular', 'Custom']).ask()
    popular_or_simplified_functions[popular_or_simplified]()


def add_base_layout() -> None:
    os.makedirs(WORKING_DIR, exist_ok=True)
    copy_files(BASE_LAYOUT, WORKING_DIR)


def add_mongo() -> None:
    MONGO_DIR = os.path.join(WORKING_DIR, DB_DIR)
    os.makedirs(MONGO_DIR, exist_ok=True)
    copy_files(DB_TREE, MONGO_DIR, ignore_dirs=[REDIS_DIR_NAME])

    # Add dnspython & pymongo to requirements.txt
    with open(os.path.join(WORKING_DIR, "requirements.txt"), "a") as f:
        f.write("\ndnspython\npymongo\n")

    docker_compose_dict['services'].update(mongo_service)
    docker_compose_dict['volumes'].update(mongo_volume)

    add_mongo_uri_to_settings()


def create_docker_compose() -> None:
    with open(os.path.join(WORKING_DIR, "docker-compose.yml"), "w") as f:
        yaml.dump(docker_compose_dict, f, default_flow_style=False, sort_keys=False)

def add_redis() -> None:
    REDIS_DIR = os.path.join(WORKING_DIR, DB_DIR)
    os.makedirs(REDIS_DIR, exist_ok=True)
    copy_files(DB_TREE, REDIS_DIR, ignore_dirs=[MONGO_DIR_NAME])

    docker_compose_dict['services'].update(redis_service)
    docker_compose_dict['volumes'].update(redis_volume)

    add_redis_uri_to_settings()


def add_cache_with_redis() -> None:
    os.makedirs(WORKING_DIR, exist_ok=True)
    copy_files(REDIS_CACHE_TREE, WORKING_DIR)


if __name__ == "__main__":
    app()



from setuptools import setup, find_packages

with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='fast-templates',
    version='0.2.0',
    packages=find_packages(),
    package_data={'': ['**/*', '.env', '.gitignore']},
    url='https://github.com/pybalt/FastAPI-TemplateWizard',
    author='pybalt',
    author_email='96897286+pybalt@users.noreply.github.com',
    description='A CLI to generate FastAPI templates',
    long_description=long_description,
    install_requires=[
        'typer',
        'questionary',
        'pyyaml'
    ],
    entry_points='''
        [console_scripts]
        fastcli=fasttemplates.main:app
    '''
)
